function [data] = make_fake_subjs(data,nListsPerSubj)
% [data] = make_fake_subjs(data,nListsPerSubj);
%
% takes the data structure created by the task free recall experiment,
% and adds a field called fake, including a specified number of fake
% subjects.  Each fake subject only has 'relabeled control' lists;
% these are created by taking the recalls from a control list and the
% presented tasks from a switch list, and combining them, allowing one
% to run permutation-based statistical tests of certain task switching
% effects.
%
% There ought to be the same number of fake subjects as real
% subjects, as they are being used as a subject by subject control;
% however, there ought to be more lists per subject than in the
% original data, to get a stable estimate for that subject. 
%
% The fake field is passed through the task_info function to
% calculate basic statistics on the lists.
%


% initialization
rand('state',sum(100*clock));
listLen = data.co.listLength;
recallLen = size(data.co.recalls,2);

if ~exist('nListsPerSubj','var')
  nListsPerSubj = 50;
end

subjlist = unique(data.co.subject);
nSubj = length(subjlist);

% erase any existing fake fields on your data structure
if isfield(data,'fake')
  these_fields = fieldnames(data.fake);
  for i = 1:length(these_fields)
    rmfield(data.fake,these_fields{i});
  end
end

% step through fake subjects

% the number of lists total in the entire experiment
% nrows = size(data.sh.subject,1);
nrows = nListsPerSubj * nSubj;

data.fake.subject = zeros(nrows,1);
for i=1:length(subjlist)
  start = ((i-1)*nListsPerSubj)+1;
  last = start + nListsPerSubj - 1;
  data.fake.subject(start:last,1) = subjlist(i);
end
data.fake.session = zeros(nrows,1);
data.fake.listType = zeros(nrows,1);
data.fake.react_time = zeros(1,listLen);
data.fake.pres_task = NaN(nrows,listLen);
data.fake.listLength = listLen;
data.fake.rec_itemnos = NaN(nrows,recallLen);
data.fake.recalls = zeros(nrows,recallLen);
data.fake.times = NaN(nrows,recallLen);
data.fake.task = NaN(nrows,recallLen);
data.fake.pres_itemnos = zeros(nrows,listLen);
data.fake.pres_subrec = zeros(nrows,listLen);

rowCount = 0;

for subj = 1:length(subjlist)
  
  s = subjlist(subj);
  % which rows for this subject
  these_rows = data.fake.subject == s;
   
  % assign session info
  data.fake.session(these_rows,1) = 1;
  % assign listType
  data.fake.listType(these_rows,1) = ones(nListsPerSubj,1)*2;      
  
  % find this subject's shift and control lists in the real data
  these_sh = find(data.sh.subject==s);
  these_co = find(data.co.subject==s);

  % now each of these_rows must be assigned recalls and times from
  % one of the same subject's control lists
  co_inds = [];
  while length(co_inds)<nListsPerSubj
    co_inds = [co_inds these_co(randperm(length(these_co)))];
  end
  co_inds = co_inds(1:nListsPerSubj);
  
  data.fake.recalls(these_rows,:) = data.co.recalls(co_inds,:);
  data.fake.times(these_rows,:) = data.co.times(co_inds,:);
  data.fake.react_time(these_rows,:) = data.co.react_time(co_inds,:);
  
  % now each of these_rows must be assigned pres_task and
  % pres_itemnos from one of the entire experiment's shift lists
  sh_inds = [];
  while length(sh_inds)<nListsPerSubj
    sh_inds = [sh_inds randperm(size(data.sh.pres_task,1))];
  end
  sh_inds = sh_inds(1:nListsPerSubj);
  
  data.fake.pres_task(these_rows,:) = data.sh.pres_task(sh_inds,:);
  data.fake.pres_itemnos(these_rows,:) = data.sh.pres_itemnos(sh_inds,:);
  
  % for each of these new lists we must now calculate the recalled
  % task order and the rec_itemnos
  these_row_inds = find(these_rows);
  for i = 1:length(these_row_inds)
    
    this_row = these_row_inds(i);
    
    % This is the critical piece of code that calculates the new
    % recalled task ordering 
    for j = 1:length(data.fake.recalls(this_row,:))      
      rec_idx = data.fake.recalls(this_row,j);
      if rec_idx > 0
	data.fake.rec_itemnos(this_row,j) = data.sh.pres_itemnos(sh_inds(i), rec_idx);
	data.fake.task(this_row,j) = data.sh.pres_task(sh_inds(i), rec_idx);	
      elseif rec_idx < 0
	data.fake.rec_itemnos(this_row,j) = -999;
	data.fake.task(this_row,j) = -999;	
      end
    end
    
  end
        
end % s nsubj

% post-processing that calculates some information about trains
data.fake = task_info(data.fake);





