function [net,env] = present_CMR(net,env,param,itemInd,taskInd,updateWeights,appendItem,isDistract,isEncoding)
% [net] = present_CMR(net,env,param,itemInd,taskInd,updateWeights,appendItem,isDistract,isEncoding);
%
% [net] = present_CMR(net,env,param,[],[],[],[],[],[]);
%
% This function simulates the presentation of an item to the
% network.  A feature is activated in the f layer, then this causes
% the c layer to update.  This script is used to variously
% simulate:
%        - the presentation of an item during study
%        - the presentation of a distraction stimulus
%        - the reinstatement of an item representation during recall
%
% net - this function assumes that the net structure has a cell array
% c_sub that contains 2 cells, one corresponding to item context
% and another corresponding to source context.
%
% param - relevant parameters are pScale, pDecay
%
% itemInd - if specified, this is the index of the feature to be
% activated in the f layer.  If not specified, the function refers to
% listIdx to determine the next feature to activate.
%
% taskInd - if specified, this is the index of the task representation
% to be activated in the f layer.  If empty, then no task information
% is activated.
%
% updateWeights - binary flag: if 1, then an association is formed
% between the f vector and the c vector.
%
% appendItem - listIdx keeps track of which items have been
% presented on the most recent list.  If this binary flag is set to
% 1, then the index of the current item is added to listIdx.
%
% isDistract - binary flag: if 1, then only the slow weights are
% used to update context.  if 1, then no associations are formed,
% only context updating.
%
%
%
% Last update: 06/16/08 by SMP
% Originally based upon TCM-A code of Per Sederberg
% 
% For information contact:
% Sean Polyn
% polyn@psych.upenn.edu
%
%


%%%%%%%%%%%%%%%%%%
% INITIALIZATION %
%%%%%%%%%%%%%%%%%%

% the default arguments, if not specified.

if isempty(updateWeights); updateWeights = 1; end
if isempty(appendItem); appendItem = 1; end
if isempty(isDistract); isDistract = 0; end
if isempty(isEncoding); isEncoding = 1; end

% a flag which triggers normalization of the input to the context
% layer
param.cINnorm = getValFromStruct(param,'cINnorm',1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ACTIVATE UNITS IN THE FEATURE LAYER % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ITEM UNITS
% determine which item is presented next.

net.f = zeros(size(net.f));
if isempty(itemInd)
  net.f(env.curStim) = 1;
  % logs the item as a list item
  if appendItem
    env.listIdx(end+1) = env.curStim;
  end
  if isEncoding
    env.curStim = env.curStim + 1;
  end
else % item ind is provided
  net.f(itemInd) = 1;
  if appendItem
    env.listIdx(end+1) = itemInd;
  end
end

% TASK UNITS
% if taskInd is empty, no task feature presented.

net.f(taskInd) = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DETERMINE THE INPUT TO THE CONTEXT LAYER %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cIN = net.w_fc * net.f;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE THE CONTEXT LAYER %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% iterate through the sub-areas of context
for i=1:length(net.c_sub)
  
  c = net.c(net.c_sub{i}.idx);
  IN = cIN(net.c_sub{i}.idx);
  
  % normalize IN if requested
  if param.cINnorm
    if sum(IN.^2)>0
      IN = IN / sqrt(IN'*IN);
    end
  end

  net.cIN(net.c_sub{i}.idx) = IN;
  
  % rho scales the current state of context (inhibition) before
  % adding in the incoming activity.
  rho = sqrt(1+(net.c_sub{i}.B^2)*((dot(c,IN)^2)-1)) - net.c_sub{i}.B*dot(c,IN);
  net.c(net.c_sub{i}.idx) = rho*c + net.c_sub{i}.B*IN;
  
end
net.c(net.c<0) = 0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSOCIATION FORMATION BETWEEN FEATURE VECTOR AND CONTEXT VECTOR %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Weight updating takes place via a simple Hebbian association.  The
% outer product of the states of the feature and context layers is
% added to the existing weight matrix.

% Associations are only formed if this is the encoding period and
% the current item is not a distractor.

% primeFact - if pScale > 0 then the learning rate for early list
% items is greater than that for later items.  Currently this primacy
% advantage only holds for the weights from c to f.  That is, primacy
% items are more likely to be retrieved (all else being equal) but
% then reinstate context like any other item.


if ~isDistract && isEncoding
  
  % calculate primacy gradient
  primeFact = (net.pScale * exp(-net.pDecay*(length(env.listIdx)-1))) + 1;
  
  % determine current learning rate
  lrate_fc = net.lrate_fc;
  lrate_cf = (net.lrate_cf .* primeFact);
  
  % weight updating
  net.w_fc = net.w_fc + ... 
      (net.c*net.f') .* lrate_fc;
  net.w_cf = net.w_cf + ... 
      (net.f*net.c') .* lrate_cf;
  
end

% UNCOMMENT FOR DEBUGGING
% fprintf('in present_CMR.\n');
% keyboard

