function [net,env] = recall_CMR(net,env,param)
% [net,env] = recall_CMR(net,env,param);
% 
% Run the CMR model through a recall period in the free-recall
% paradigm.
%
% env.timer - a structure that keeps track of the total time in the
% broader recall period.  The structure must contain the following
% fields: 
%   env.timer.recTime - total milliseconds in the recall period
%   env.timer.timePassed - number of milliseconds that have passed already
%
% 


%%%%%%%%%%%%%%%%%%
% INITIALIZATION %
%%%%%%%%%%%%%%%%%%

rand('state',sum(100*clock));
visualize_item_recall = getValFromStruct(param,'visualize_item_recall',0);
LL = length(env.listIdx);

% the connections from the feature units to the accumulators
net.w_fa = getValFromStruct(net,'w_fa',eye(length(env.listIdx)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DETERMINE THE INPUT TO THE FEATURE LAYER %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% the input from context to feature layer
f_in = net.w_cf * net.c;

% prepare this to be sent to the accumulators
f_in = f_in(env.listIdx);
in = net.w_fa * f_in;

% prepare a column vector of the already-recalled items
param.recalled = logical(env.recalled');

decision_idx = env.listIdx;

%%%%%%%%%%%%%%%%%%%%%%%%
% RUN THE ACCUMULATORS %
%   RECALL AN ITEM     %
%%%%%%%%%%%%%%%%%%%%%%%%

% generate the noise matrix before the accumulation starts
maxcycles = (env.timer.recTime - env.timer.timePassed) / param.dt;
noise = randn(length(in),maxcycles);

[decision_out,time] = decision_accum(param,in,noise);
out_idx = decision_idx(decision_out);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VISUALIZATION OF THE RECALL PROCESS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if visualize_item_recall == 1
  f = sfigure(1);
  clf;
  % plot the inputs to the accumulators
  plot(env.allPresentedIdx,in,'.r-','LineWidth',2);
  hold on;
  % plot the current state of context
  plot(1:length(net.c),net.c,'.b-','LineWidth',2);
  grid on; box on;
  % plot the recalled items
  tmp1 = zeros(1,length(net.f));
  tmp1(env.listIdx(logical(env.recalled))) = -0.01;
  tmp1(tmp1==0)=-10;
  % the about-to-be-recalled item
  a = plot(tmp1,'og','MarkerSize',10);
  set(a,'MarkerFaceColor',[0 1 0]);
  tmp2 = ones(1,length(net.f))*-10;
  tmp2(out_idx) = -0.01;
  % all the other recalled items
  b = plot(tmp2,'or','MarkerSize',10);
  set(b,'MarkerFaceColor',[1 0 0]);
  ymax = max(in);
  axis([0 40 -0.05 ymax+0.1]);
  keyboard  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MANAGING THE ACCUMULATOR OUTPUT %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% update the clock
env.timer.timePassed = env.timer.timePassed + time;

% update the recall counter 
env.recallCount = env.recallCount + 1;

% different ways of recording the most recent recall
env.recalled(decision_out) = 1;
env.lastRecPos = decision_out;
env.lastRecIdx = out_idx;

% reinstate the recalled item in the f layer
net.f = zeros(size(net.f));
net.f(env.lastRecIdx) = 1;

